==> tib-0.6.3.tgz: package/lib/tib-chunk.js <==
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

function _interopNamespace(e) {
  if (e && e.__esModule) { return e; } else {
    var n = {};
    if (e) {
      Object.keys(e).forEach(function (k) {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      });
    }
    n['default'] = e;
    return n;
  }
}

const path = _interopDefault(require('path'));
const onExit = _interopDefault(require('signal-exit'));
const kill = _interopDefault(require('tree-kill'));
const vueTemplateCompiler = require('vue-template-compiler');
const core = require('@babel/core');
const parser = require('@babel/parser');

class BrowserError extends Error {
  constructor(classInstance, message, ...params) {
    let className;
    if (!message) {
      message = classInstance;
      className = 'BrowserError';
    } else if (typeof classInstance === 'object') {
      className = classInstance.constructor.name;
    } else if (typeof classInstance === 'string') {
      className = classInstance;
    }

    // Pass remaining arguments (including vendor specific ones) to parent constructor
    super(`${className}: ${message}`, ...params);

    // Maintains proper stack trace for where our error was thrown (only available on V8)
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, BrowserError);
    }
  }
}

// https://www.browserstack.com/automate/node#configure-capabilities
// https://wiki.saucelabs.com/display/DOCS/Platform+Configurator

const browsers = [
  'chrome',
  'firefox',
  'ie',
  'edge',
  'safari'
];

const browserOptions = [
  'headless',
  'xvfb',
  'staticserver'
];

const drivers = [
  'puppeteer',
  'puppeteer-core',
  'selenium'
];

const providers = [
  'browserstack',
  'saucelabs'
];

const browserVariants = {
  local: ['browserstack'],
  core: ['puppeteer']
};

const os = [
  'windows',
  'linux',
  'mac',
  'macos',
  'macosx',
  'osx',
  'android',
  'ios'
];

function abstractGuard(className, { name } = {}) {
  if (className === name) {
    throw new BrowserError(`Do not use abstract class '${className}' directly`)
  }
}

function isMockedFunction(fn, fnName) {
  return fn.name !== fnName
}

// TODO: add more test framework checks like sinon?
function disableTimers() {
  // find Jest fingerprint
  if (process.env.JEST_WORKER_ID) {
    try {
      jest.useFakeTimers();
    } catch (e) {
      /* istanbul ignore next */
      throw new BrowserError(`Enabling fake timers failed: ${e.message}`)
    }
  }
}

function enableTimers() {
  // find Jest fingerprint
  if (process.env.JEST_WORKER_ID) {
    try {
      // call useFakeTimers first as it seems there is no way to determine
      // if fake timers are used at all and otherwise runOnlyPendingTimers
      // will fail with a warning from within Jest
      // -> Disabled because we probably dont care about pending timers
      // jest.useFakeTimers()
      // jest.runOnlyPendingTimers()
      jest.useRealTimers();
    } catch (e) {
      /* istanbul ignore next */
      throw new BrowserError(`Enabling real timers failed: ${e.message}`)
    }
  }
}

const fnCache = {};
function parseFunction(fn, presetOptions = {}) {
  if (typeof fn !== 'function') {
    throw new BrowserError(`parseFunction expects the first argument to be a function, received '${typeof fn}' instead`)
  }

  const presetString = JSON.stringify(presetOptions);
  const fnString = fn.toString();
  if (fnCache[fnString] && fnCache[fnString][presetString]) {
    return fnCache[fnString][presetString]
  }

  const parsed = {};
  let ast = parser.parseExpression(fnString);
  parsed.args = ast.params.map(p => p.name);

  ast = ast.body;

  /* transform can only transform a program and a program is an Array of
   * Statements.
   * The body of an arrow function like 'arg => arg' does not
   * contain a Statement, so we add a block & return statement in that case.
   * The return statement is needed later when we create a new Function and the
   * block statement helps so we can always call slice(1, -1) below to
   * retrieve the real function body
   */
  if (!ast.type.includes('Statement')) {
    ast = core.types.blockStatement([core.types.returnStatement(ast)]);
  }

  const transpiled = core.transformFromAstSync(
    core.types.file(core.types.program([ast])),
    undefined,
    {
      sourceType: 'script',
      presets: [
        ['@babel/preset-env', presetOptions]
      ]
    }
  );

  // remove block statement needed to transform & trim block whitespace
  parsed.body = transpiled.code.slice(1, -1).trim();

  if (!fnCache[fnString]) {
    fnCache[fnString] = {};
  }

  fnCache[fnString][presetString] = parsed;

  return parsed
}

function getDefaultHtmlCompiler() {
  return html => vueTemplateCompiler.compile(html).ast
}

function loadDependency(dependency) {
  try {
    return new Promise(function (resolve) { resolve(_interopNamespace(require(dependency))); }).then(m => m.default || m)
  } catch (e) {
    throw new BrowserError(`Could not import the required dependency '${dependency}'
(error: ${e.message})

Please install the dependency with:
$ npm install ${dependency}
or
$ yarn add ${dependency}
    `)
  }
}

function getBrowserConfigFromString(browserString) {
  const browserConfig = {};

  browserString
    .split('/')
    .filter(s => !!s)
    .map(s => s.toLowerCase()
      // replace first occurence with =
      // -> Chrome 39 => chrome=39
      // -> Mac OS X=High Sierra => Mac OS X=High Sierra
      .replace(/([\s:])/, (m, $1, o, s) => (s.includes('=') ? $1 : '='))
    )
    .forEach((s) => {
      let [key, value] = s.split('=', 2); // eslint-disable-line prefer-const
      key = key.replace(/\s+/, '');

      if (browsers.includes(key)) {
        browserConfig.browser = key;

        if (value) {
          browserConfig.browserVersion = value.replace(/\s+/g, '');
        }
        return
      }

      if (browserOptions.includes(key)) {
        browserConfig[key] = typeof value === 'undefined' ? true : value;
        return
      }

      if (os.includes(key)) {
        browserConfig.os = key;

        if (value) {
          browserConfig.osVersion = value;
        }
        return
      }

      if (drivers.includes(key)) {
        browserConfig.driver = key;
        return
      }

      if (providers.includes(key)) {
        browserConfig.provider = key;
        return
      }

      if (browserVariants[key]) {
        browserConfig.browserVariant = key;
        return
      }

      if (key === 'device') {
        browserConfig.device = value;
        return
      }

      if (!value) {
        const screenResolution = key.split(/(\d+)x(\d+)/i);
        if (screenResolution && screenResolution.length > 1) {
          browserConfig.window = {
            width: parseInt(screenResolution[1]),
            height: parseInt(screenResolution[2])
          };
        }
      }
    });

  return browserConfig
}

function getBrowserImportFromConfig(browserConfig) {
  const importPath = [];

  if (browserConfig.browser) {
    importPath.push(browserConfig.browser);
  }

  if (browserConfig.driver && !browserConfig.provider) {
    importPath.push(browserConfig.driver);
  }

  if (browserConfig.provider) {
    importPath.push(browserConfig.provider);
  }

  if (browserConfig.browserVariant) {
    importPath.push(browserConfig.browserVariant);
  }

  return importPath.join('/')
}

const consola = console; // eslint-disable-line no-console

let PID;

class BrowserStackLocal {
  static async loadDriver() {
    if (BrowserStackLocal.driver) {
      return
    }

    const browserstack = await loadDependency('browserstack-local');
    BrowserStackLocal.driver = new browserstack.Local();
  }

  static async start(config = {}) {
    // TODO: support webserver
    if (!config.folder) {
      config.folder = path.resolve(process.cwd());
    }

    await BrowserStackLocal.loadDriver();

    // util.promisify doesnt work due to this binding and
    // .bind(local) didnt work either
    return new Promise((resolve, reject) => {
      BrowserStackLocal.driver.start(config, (error) => {
        if (error) {
          reject(error);
        }

        onExit(() => BrowserStackLocal.stop());

        PID = BrowserStackLocal.driver.pid;
        resolve(PID);
      });
    })
  }

  static stop(pid) {
    pid = pid || (BrowserStackLocal.driver && BrowserStackLocal.driver.pid) || PID;

    if (!BrowserStackLocal.driver || !pid) {
      consola.warn(`Stop called but browserstack-local was not started`);
      return
    }

    return new Promise((resolve, reject) => {
      // local.stop is buggy, it doesnt kill anything and takes forever
      // after looking at the local.stop implementation tree-kill does
      // practically the same
      kill(pid, 'SIGTERM', (error) => {
        if (error) {
          reject(error);
        }

        resolve();
      });
    })
  }
}

exports.BrowserError = BrowserError;
exports.BrowserStackLocal = BrowserStackLocal;
exports.abstractGuard = abstractGuard;
exports.disableTimers = disableTimers;
exports.enableTimers = enableTimers;
exports.getBrowserConfigFromString = getBrowserConfigFromString;
exports.getBrowserImportFromConfig = getBrowserImportFromConfig;
exports.getDefaultHtmlCompiler = getDefaultHtmlCompiler;
exports.isMockedFunction = isMockedFunction;
exports.loadDependency = loadDependency;
exports.parseFunction = parseFunction;

==> tib-0.6.5.tgz: package/lib/tib-chunk.js <==
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

function _interopNamespace(e) {
  if (e && e.__esModule) { return e; } else {
    var n = {};
    if (e) {
      Object.keys(e).forEach(function (k) {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      });
    }
    n['default'] = e;
    return n;
  }
}

const path = _interopDefault(require('path'));
const onExit = _interopDefault(require('signal-exit'));
const kill = _interopDefault(require('tree-kill'));
const vueTemplateCompiler = require('vue-template-compiler');
const core = require('@babel/core');
const parser = require('@babel/parser');

class BrowserError extends Error {
  constructor(classInstance, message, ...params) {
    let className;
    if (!message) {
      message = classInstance;
      className = 'BrowserError';
    } else if (typeof classInstance === 'object') {
      className = classInstance.constructor.name;
    } else if (typeof classInstance === 'string') {
      className = classInstance;
    }

    // Pass remaining arguments (including vendor specific ones) to parent constructor
    super(`${className}: ${message}`, ...params);

    // Maintains proper stack trace for where our error was thrown (only available on V8)
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, BrowserError);
    }
  }
}

// https://www.browserstack.com/automate/node#configure-capabilities
// https://wiki.saucelabs.com/display/DOCS/Platform+Configurator

const browsers = [
  'chrome',
  'firefox',
  'ie',
  'edge',
  'safari'
];

const browserOptions = [
  'headless',
  'xvfb',
  'staticserver'
];

const drivers = [
  'puppeteer',
  'puppeteer-core',
  'selenium'
];

const providers = [
  'browserstack',
  'saucelabs'
];

const browserVariants = {
  local: ['browserstack'],
  core: ['puppeteer']
};

const os = [
  'windows',
  'linux',
  'mac',
  'macos',
  'macosx',
  'osx',
  'android',
  'ios'
];

function abstractGuard(className, { name } = {}) {
  if (className === name) {
    throw new BrowserError(`Do not use abstract class '${className}' directly`)
  }
}

function isMockedFunction(fn, fnName) {
  return fn.name !== fnName
}

// TODO: add more test framework checks like sinon?
function disableTimers() {
  // find Jest fingerprint
  if (process.env.JEST_WORKER_ID) {
    try {
      jest.useFakeTimers();
    } catch (e) {
      /* istanbul ignore next */
      throw new BrowserError(`Enabling fake timers failed: ${e.message}`)
    }
  }
}

function enableTimers() {
  // find Jest fingerprint
  if (process.env.JEST_WORKER_ID) {
    try {
      // call useFakeTimers first as it seems there is no way to determine
      // if fake timers are used at all and otherwise runOnlyPendingTimers
      // will fail with a warning from within Jest
      // -> Disabled because we probably dont care about pending timers
      // jest.useFakeTimers()
      // jest.runOnlyPendingTimers()
      jest.useRealTimers();
    } catch (e) {
      /* istanbul ignore next */
      throw new BrowserError(`Enabling real timers failed: ${e.message}`)
    }
  }
}

const fnCache = {};
function parseFunction(fn, presetOptions = {}) {
  if (typeof fn !== 'function') {
    throw new BrowserError(`parseFunction expects the first argument to be a function, received '${typeof fn}' instead`)
  }

  const presetString = JSON.stringify(presetOptions);
  const fnString = fn.toString();
  if (fnCache[fnString] && fnCache[fnString][presetString]) {
    return fnCache[fnString][presetString]
  }

  const parsed = {};
  let ast = parser.parseExpression(fnString);
  parsed.args = ast.params.map(p => p.name);

  ast = ast.body;

  /* transform can only transform a program and a program is an Array of
   * Statements.
   * The body of an arrow function like 'arg => arg' does not
   * contain a Statement, so we add a block & return statement in that case.
   * The return statement is needed later when we create a new Function and the
   * block statement helps so we can always call slice(1, -1) below to
   * retrieve the real function body
   */
  if (!ast.type.includes('Statement')) {
    ast = core.types.blockStatement([core.types.returnStatement(ast)]);
  }

  const transpiled = core.transformFromAstSync(
    core.types.file(core.types.program([ast])),
    undefined,
    {
      sourceType: 'script',
      presets: [
        ['@babel/preset-env', presetOptions]
      ]
    }
  );

  // remove block statement needed to transform & trim block whitespace
  parsed.body = transpiled.code.slice(1, -1).trim();

  if (!fnCache[fnString]) {
    fnCache[fnString] = {};
  }

  fnCache[fnString][presetString] = parsed;

  return parsed
}

function getDefaultHtmlCompiler() {
  return html => vueTemplateCompiler.compile(html).ast
}

function loadDependency(dependency) {
  try {
    return new Promise(function (resolve) { resolve(_interopNamespace(require(dependency))); }).then(m => m.default || m)
  } catch (e) {
    throw new BrowserError(`Could not import the required dependency '${dependency}'
(error: ${e.message})

Please install the dependency with:
$ npm install ${dependency}
or
$ yarn add ${dependency}
    `)
  }
}

function getBrowserConfigFromString(browserString) {
  const browserConfig = {};

  browserString
    .split('/')
    .filter(s => !!s)
    .map(s => s.toLowerCase()
      // replace first occurence with =
      // -> Chrome 39 => chrome=39
      // -> Mac OS X=High Sierra => Mac OS X=High Sierra
      .replace(/([\s:])/, (m, $1, o, s) => (s.includes('=') ? $1 : '='))
    )
    .forEach((s) => {
      let [key, value] = s.split('=', 2); // eslint-disable-line prefer-const
      key = key.replace(/\s+/, '');

      if (browsers.includes(key)) {
        browserConfig.browser = key;

        if (value) {
          browserConfig.browserVersion = value.replace(/\s+/g, '');
        }
        return
      }

      if (browserOptions.includes(key)) {
        browserConfig[key] = typeof value === 'undefined' ? true : value;
        return
      }

      if (os.includes(key)) {
        browserConfig.os = key;

        if (value) {
          browserConfig.osVersion = value;
        }
        return
      }

      if (drivers.includes(key)) {
        browserConfig.driver = key;
        return
      }

      if (providers.includes(key)) {
        browserConfig.provider = key;
        return
      }

      if (browserVariants[key]) {
        browserConfig.browserVariant = key;
        return
      }

      if (key === 'device') {
        browserConfig.device = value;
        return
      }

      if (!value) {
        const screenResolution = key.split(/(\d+)x(\d+)/i);
        if (screenResolution && screenResolution.length > 1) {
          browserConfig.window = {
            width: parseInt(screenResolution[1]),
            height: parseInt(screenResolution[2])
          };
        }
      }
    });

  return browserConfig
}

function getBrowserImportFromConfig(browserConfig) {
  const importPath = [];

  if (browserConfig.browser) {
    importPath.push(browserConfig.browser);
  }

  if (browserConfig.driver && !browserConfig.provider) {
    importPath.push(browserConfig.driver);
  }

  if (browserConfig.provider) {
    importPath.push(browserConfig.provider);
  }

  if (browserConfig.browserVariant) {
    importPath.push(browserConfig.browserVariant);
  }

  return importPath.join('/')
}

const consola = console; // eslint-disable-line no-console

let PID;

class BrowserStackLocal {
  static async loadDriver() {
    if (BrowserStackLocal.driver) {
      return
    }

    const browserstack = await loadDependency('browserstack-local');
    BrowserStackLocal.driver = new browserstack.Local();
  }

  static async start(config = {}) {
    // TODO: support webserver
    if (!config.folder && config.folder !== false) {
      config.folder = path.resolve(process.cwd());
    }

    await BrowserStackLocal.loadDriver();

    // util.promisify doesnt work due to this binding and
    // .bind(local) didnt work either
    return new Promise((resolve, reject) => {
      BrowserStackLocal.driver.start(config, (error) => {
        if (error) {
          reject(error);
        }

        onExit(() => BrowserStackLocal.stop());

        PID = BrowserStackLocal.driver.pid;
        resolve(PID);
      });
    })
  }

  static stop(pid) {
    pid = pid || (BrowserStackLocal.driver && BrowserStackLocal.driver.pid) || PID;

    if (!BrowserStackLocal.driver || !pid) {
      consola.warn(`Stop called but browserstack-local was not started`);
      return
    }

    return new Promise((resolve, reject) => {
      // local.stop is buggy, it doesnt kill anything and takes forever
      // after looking at the local.stop implementation tree-kill does
      // practically the same
      kill(pid, 'SIGTERM', (error) => {
        if (error) {
          reject(error);
        }

        resolve();
      });
    })
  }
}

exports.BrowserError = BrowserError;
exports.BrowserStackLocal = BrowserStackLocal;
exports.abstractGuard = abstractGuard;
exports.disableTimers = disableTimers;
exports.enableTimers = enableTimers;
exports.getBrowserConfigFromString = getBrowserConfigFromString;
exports.getBrowserImportFromConfig = getBrowserImportFromConfig;
exports.getDefaultHtmlCompiler = getDefaultHtmlCompiler;
exports.isMockedFunction = isMockedFunction;
exports.loadDependency = loadDependency;
exports.parseFunction = parseFunction;
